@@ -44,6 +44,20 @@ HELP_FILE = "/usr/share/doc/topshelf/topshelf.docbook"
 
 ## Global utilities
 
+def update_background(widget, backgroundtype, color, pixmap):
+	# Needed for every widget on the panel
+	widget.set_style(None)
+	rc_style = gtk.RcStyle()
+	widget.modify_style(rc_style)
+	widget.modify_bg(gtk.STATE_NORMAL, color)
+
+	if backgroundtype == gnomeapplet.PIXMAP_BACKGROUND:
+		style = widget.get_style().copy()
+		style.bg_pixmap[gtk.STATE_NORMAL] = pixmap
+		widget.set_style(style);
+	elif backgroundtype == gnomeapplet.COLOR_BACKGROUND:
+		widget.modify_bg(gtk.STATE_NORMAL, color)
+
 def popup_warning(window, message):
 	dlg = gtk.MessageDialog(parent         = window,
 	                        flags          = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
@@ -52,10 +66,12 @@ def popup_warning(window, message):
 	                        type           = gtk.MESSAGE_WARNING)
 
 	window.present() # Ensure we are on top, for our losing keep_above to not hide us
-	window.set_keep_above(False)
+	if window is not None:
+		window.set_keep_above(False)
 	result = dlg.run()
 	dlg.destroy()
-	window.set_keep_above(True)
+	if window is not None:
+		window.set_keep_above(True)
 
 def popup_question(window, message):
 	dlg = gtk.MessageDialog(parent         = window,
@@ -65,10 +81,12 @@ def popup_question(window, message):
 	                        type           = gtk.MESSAGE_QUESTION)
 
 	window.present() # Ensure we are on top, for our losing keep_above to not hide us
-	window.set_keep_above(False)
+	if window is not None:
+		window.set_keep_above(False)
 	result = dlg.run()
 	dlg.destroy()
-	window.set_keep_above(True)
+	if window is not None:
+		window.set_keep_above(True)
 
 	if result == gtk.RESPONSE_YES:
 		return True
@@ -150,12 +168,14 @@ class PersistentData: ## Legacy!
 		self.h = -1
 
 current_files = "current_files" # A constant, really...
+add_folder    = "add_folder"
 
-DEFAULT_VALUES = { current_files: [],
-                   'x': 400,   # For standalone app
-                   'y': 300,   # For standalone app
-                   'w': 600,
-                   'h': 350  }
+DEFAULT_VALUES = { current_files : [],
+                   add_folder    : os.path.expanduser("~"),
+                   'x' : 400,   # For standalone app
+                   'y' : 300,   # For standalone app
+                   'w' : 600,
+                   'h' : 350  }
 
 class ModernPersistentData:
 	def __init__(self):
@@ -204,27 +224,28 @@ class TopShelf:
 		self.applet = applet
 		self.applet.ts = self # Needed for standalone
 
+#		self.applet.set_flags(gnomeapplet.EXPAND_MAJOR) # TODO: Needed?
+
 		self.standalone_window = standalone_window
 
 		self.window = None
 
 		self.pressed  = False
-		self.resizing = False
 
-		self.panel_pb     = gtk.gdk.pixbuf_new_from_file("/usr/share/pixmaps/topshelf-24.png")
-		self.panel_off_pb = gtk.gdk.pixbuf_new_from_file("/usr/share/pixmaps/topshelf-24-off.png")
+		try:
+			self.applet.set_tooltip_text("TopShelf")
+		except AttributeError:
+			pass # No tooltips, not the end of the world. Only python-gtk2 2.12 has them, i.e., Gutsy and above
+
+		self.applet.connect("button_press_event", self.on_button_press)
+		self.applet.drag_dest_set(gtk.DEST_DEFAULT_ALL, [('text/uri-list', 0, 81)], gtk.gdk.ACTION_LINK)
+		self.applet.connect("drag_data_received", self.on_drag_data_received)
+		self.applet.connect("key_press_event",    self.on_key_press)
+		self.applet.connect("change_background",  self.on_change_background)
+		self.applet.connect("change-size",        self.on_change_size)
 
 		self.panel_image = gtk.Image()
-		self.panel_image.set_from_pixbuf(self.panel_pb)
-		self.panel_image.set_tooltip_text("TopShelf")
-		self.eventbox = gtk.EventBox()
-		self.eventbox.connect("button-press-event", self.do_button_press)
-		self.eventbox.drag_dest_set(gtk.DEST_DEFAULT_ALL, [('text/uri-list', 0, 81)], gtk.gdk.ACTION_LINK)
-		self.eventbox.connect("drag-data-received", self.do_drag_data_received)
-		self.eventbox.connect("key-press-event", self.do_key_press)
-
-		self.eventbox.add(self.panel_image)
-		self.applet.add(self.eventbox)
+		self.applet.add(self.panel_image)
 		self.applet.show_all()
 
 		# full-path, position, short-filename, mime-type-icon, color, strikethrough
@@ -240,36 +261,82 @@ class TopShelf:
 			self.standalone_window.connect("configure-event", self.standalone_window_configure_event)
 			self.standalone_window.set_keep_above(True)
 
+		# Initial size
+		self.size = self.applet.get_size()
+		self.on_change_size(self.applet, self.size)
+
+
+	# Init and setup
+
+	def get_pixbufs(self, size):
+		self.panel_pbs = []
+		for pb_type in ["", "-off"]: # On, then off
+			# With help from the Deskbar applet. Thanks!
+			image_name = "/usr/share/pixmaps/topshelf-"
+			if size <= 32:
+				image_name += "24" + pb_type + ".png"
+				size = 24
+			else:
+				image_name += "svg" + pb_type + ".svg"
+				size -= 6
+
+			self.panel_pbs.append(gtk.gdk.pixbuf_new_from_file_at_size(image_name, size, size))
+
+	def set_image(self):
+		if not self.pressed:
+			self.panel_image.set_from_pixbuf(self.panel_pbs[0])
+		else:		
+			self.panel_image.set_from_pixbuf(self.panel_pbs[1])
+
+
 	# Events
 
-	def do_button_toggle(self, param1=None):
+	def on_change_background(self, applet, backgroundtype, color, pixmap):
+		update_background(applet, backgroundtype, color, pixmap)
+
+	def on_change_size(self, applet, size):
+		self.get_pixbufs(size)
+		self.applet.set_size_request(size, size)
+		self.set_image()
+
+		self.size = size
+
+
+	# Events
+
+	def on_button_toggle(self, param1=None):
 		if not self.pressed:
 			self.show_window()
 		else:
 			self.hide_window()
 
-	def do_button_press(self, btn, event):
+	def on_button_press(self, btn, event):
+		self.on_change_size(self.applet, self.applet.get_size())
 		if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
-			if not self.resizing:
-				self.do_button_toggle()
-			else:
-				self.do_stop_resizing()
+			self.on_button_toggle()
 		elif event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
-			btn.emit_stop_by_name("button_press_event")
 			propxml="""
 				<popup name="button3">
 				<menuitem name="Item 3" verb="Help"  label="_Help" pixtype="stock" pixname="gtk-help"/>
 				<menuitem name="Item 4" verb="About" label="_About" pixtype="stock" pixname="gtk-about"/>
 				</popup>"""
-			verbs = [("About", self.do_about),
-			         ("Help",  self.do_help)  ]
+			verbs = [("About", self.on_about),
+			         ("Help",  self.on_help)  ]
 			self.applet.setup_menu(propxml, verbs, None)
 
-	def do_files_button_press(self, btn, event):
+	def on_files_button_press(self, btn, event):
 		if event.button == 3:
-			model, iter = self.get_selected()
-			if iter is not None:
+			data = self.treeview.get_path_at_pos(int(event.x), int(event.y))
+			if data is None:
+				return True
+
+			path = data[0]
+			self.treeview.grab_focus()
+			self.treeview.set_cursor(path)
+			model, iters = self.get_selected()
+			if len(iters) > 0:
 				self.popup_menu.popup(None, None, None, event.button, event.time)
+				return True
 
 		return False
 
@@ -294,23 +361,25 @@ class TopShelf:
 
 			self.pickle()
 
-	def do_cursor_change(self, param1):
-		model, iter = self.get_selected()
-		self.check_file(iter)
+	def on_cursor_change(self, param1):
+		model, iters = self.get_selected()
+		for iter in iters:
+			self.check_file(iter)
 
 	# Actions
 
-	def do_about(self, param1=None, param2=None):
-		self.window.set_keep_above(False) # So we are not concealed!
+	def on_about(self, param1=None, param2=None):
+		if self.window is not None:
+			self.window.set_keep_above(False) # So we are not concealed!
 
-		logo_pb = gtk.gdk.pixbuf_new_from_file("/usr/share/pixmaps/topshelf-64.png")
+		logo_pb = gtk.gdk.pixbuf_new_from_file_at_size("/usr/share/pixmaps/topshelf-svg.svg", 64, 64)
 
 		dlg = gtk.AboutDialog()
 		dlg.set_icon(logo_pb)
 
 		dlg.set_name("TopShelf")
 		dlg.set_program_name("TopShelf")
-		dlg.set_version("0.2.1")
+		dlg.set_version("0.3")
 		dlg.set_comments("Currently Important Files Applet")
 		dlg.set_authors(["Alon Zakai (Kripken)"])
 		dlg.set_license("TopShelf is free software; you can redistribute it and/or\nmodify it under the terms of the GNU General Public License\nas published by the Free Software Foundation; either version\n2 of the License, or (at your option) any later version.")
@@ -321,13 +390,15 @@ class TopShelf:
 		dlg.run()
 		dlg.destroy()
 
-		self.window.set_keep_above(True)
+		if self.window is not None:
+			self.window.set_keep_above(True)
 
 		return False
 
-	def do_open(self, param1=None, param2=None, param3=None):
-		model, iter = self.get_selected()
-		if iter is not None:
+	def on_open(self, param1=None, param2=None, param3=None):
+		model, iters = self.get_selected()
+		# TODO: open them all in one command (good for xmms etc.). Sadly gnome-open doesn't allow multiple files...
+		for iter in iters:
 			self.check_file(iter) # Update color depending on existence of file. Not needed if a thread does this!
 
 			filename = model.get_value(iter, 0)
@@ -338,9 +409,9 @@ class TopShelf:
 
 		return False
 
-	def do_open_folder(self, param1=None, param2=None, param3=None):
-		model, iter = self.get_selected()
-		if iter is not None:
+	def on_open_folder(self, param1=None, param2=None, param3=None):
+		model, iters = self.get_selected()
+		for iter in iters:
 			self.check_file(iter) # Update color depending on existence of file. Not needed if a thread does this!
 
 			dirname = os.path.dirname(model.get_value(iter, 0))
@@ -351,67 +422,76 @@ class TopShelf:
 
 		return False
 
-	def do_move_up(self, param1=None):
-		model, iter = self.get_selected()
-		if iter is not None:
+	def on_move_up(self, param1=None):
+		model, iters = self.get_selected()
+		for iter in iters:
 			old_val = model.get(iter, 1)[0]
 			new_val = old_val - 1
 			self.swap_file_position(model, iter, old_val, new_val)
 
 		return False
 
-	def do_move_down(self, param1=None):
-		model, iter = self.get_selected()
-		if iter is not None:
+	def on_move_down(self, param1=None):
+		model, iters = self.get_selected()
+		for iter in iters:
 			old_val = model.get(iter, 1)[0]
 			new_val = old_val + 1
 			self.swap_file_position(model, iter, old_val, new_val)
 
 		return False
 
-	def do_add(self, wgt):
+	def on_add(self, wgt):
 		self.window.set_keep_above(False) # So we are not concealed
 		chooser = gtk.FileChooserDialog("Select a file to add to TopShelf:",
 		                                self.window,
 		                                gtk.FILE_CHOOSER_ACTION_OPEN,
 		                                buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
 
+		chooser.set_select_multiple(True)
+		chooser.set_current_folder(self.persistent_data.get(add_folder))
+
 		response = chooser.run()
 		if response == gtk.RESPONSE_OK:
-			filename = chooser.get_filenames()
+			self.persistent_data.set(add_folder, chooser.get_current_folder())
+			filenames = chooser.get_filenames()
 		else:
-			response == None
+			filenames = None
 
 		chooser.destroy()
 
 		self.window.set_keep_above(True)
 
-		if filename is not None:
-			self.add_file(filename[0])
+		if filenames is not None:
+			for filename in filenames:
+				self.add_file(filename)
 
 		return False
 
-	def do_remove(self, wgt):
-		model, iter = self.get_selected()
-		if iter is not None:
+	def on_remove(self, wgt=None):
+		model, iters = self.get_selected()
+		if len(iters) > 0:
 			if popup_question(self.window,
-			                  "Are you sure you want to remove the file '" + model.get(iter, 2)[0] +
-			                  "' from the list? (The actual file will not be touched)"):
-				old_position = model.get(iter, 1)[0]
-				self.files_store.remove(iter)
-				self.remove_file_position(model, old_position)
+			                  "Are you sure you want to remove the selected files from the list? (The actual files will not be touched)"):
+				filenames = []
+				for iter in iters:
+					filenames.append(model.get(iter, 0)[0])
+				for filename in filenames:
+					iter = self.get_file_iter_by_filename(filename)
+					old_position = model.get(iter, 1)[0]
+					self.files_store.remove(iter)
+					self.remove_file_position(model, old_position)
 
 				self.pickle()
 
 		return False
 
-	def do_drag_data_received(self, widget, drag_context, x, y, selection_data, info, timestamp):
+	def on_drag_data_received(self, widget, drag_context, x, y, selection_data, info, timestamp):
 		uris = selection_data.data.strip().split()
 		for uri in uris:
 			filename = urllib.url2pathname(uri)
 			self.add_file(filename)
 
-	def do_package(self, param1=None):
+	def on_package(self, param1=None):
 		if self.get_n_files() == 0:
 			return
 
@@ -443,7 +523,7 @@ class TopShelf:
 
 		return False
 
-	def do_help(self, param1=None, param2=None, param3=None):
+	def on_help(self, param1=None, param2=None, param3=None):
 		self.hide_window()
 		ret = subprocess.call(HELP_APP + " '" + HELP_FILE + "'", shell=True)
 		if ret != 0:
@@ -451,11 +531,13 @@ class TopShelf:
 
 		return False
 
-	def do_key_press(self, widget, event):
+	def on_key_press(self, widget, event):
 		if event.keyval == 65307:
 			self.hide_window()
 		elif event.keyval == 65535:
-			self.do_remove(None)
+			self.on_remove()
+		elif event.keyval == 65474:
+			self.check_all_files()
 
 
 	# Persistency
@@ -510,8 +592,6 @@ class TopShelf:
 		if self.window is None:
 			self.create_window()
 
-		if not self.resizing:
-			self.panel_image.set_from_pixbuf(self.panel_off_pb)
 		self.pressed = True
 
 		# These four commands - present, keep-above, present, position - are very odd. But they work, in THIS order.
@@ -520,9 +600,11 @@ class TopShelf:
 		self.window.present()
 		self.position_window() # Needed here because of odd behavior otherwise (after adding, hide, show, it moved)
 
-		self.check_files_thread() # One manual operation, then later as a background thread
+		self.check_all_files() # One manual operation, then later as a background thread
 #		gobject.timeout_add(2000, self.check_files_thread) No thread for now, since it wastes CPU. Later, add an option to do this.
 
+		self.set_image()
+
 	def hide_window(self, param1=False, param2=False):
 		if self.window is None:
 			return True
@@ -532,9 +614,10 @@ class TopShelf:
 		while gtk.events_pending():
 			gtk.main_iteration() # Ensure the hide actually happens before we show e.g. Help windows
 
-		self.panel_image.set_from_pixbuf(self.panel_pb)
 		self.pressed = False
 
+		self.set_image()
+
 		return True # Stop delete events
 
 	def create_window(self):
@@ -544,18 +627,20 @@ class TopShelf:
 		self.window.connect("delete-event",       self.hide_window)
 		self.window.connect("window-state-event", self.window_state_event)
 		self.window.connect("configure-event",    self.configure_event)
-		self.window.connect("key-press-event",    self.do_key_press)
+		self.window.connect("key-press-event",    self.on_key_press)
 
 		self.window.set_keep_above(True)
 		self.window.resize(self.persistent_data.get('w'), self.persistent_data.get('h'))
 
 		self.window.drag_dest_set(gtk.DEST_DEFAULT_ALL, [('text/uri-list', 0, 81)], gtk.gdk.ACTION_LINK)
-		self.window.connect("drag-data-received", self.do_drag_data_received)
+		self.window.connect("drag-data-received", self.on_drag_data_received)
 
 		# Main view
 
 		self.treeview = gtk.TreeView()
 		self.treeview.set_model(self.files_store)
+		self.treeview.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+
 		col = self.append_column(self.treeview, gtk.TreeViewColumn("#",    gtk.CellRendererText(),   text  =1), 1)
 		tmp = self.append_column(self.treeview, gtk.TreeViewColumn("",     gtk.CellRendererPixbuf(), pixbuf=3))
 		tmp = self.append_column(self.treeview, gtk.TreeViewColumn("File", gtk.CellRendererText(),   text  =2), 2,
@@ -563,9 +648,10 @@ class TopShelf:
 		                         strikethrough_id = FILE_STORE_STRIKETHROUGH)
 
 		col.clicked() # Set default sort
-		self.treeview.connect("row-activated" ,     self.do_open)
-		self.treeview.connect("cursor-changed",     self.do_cursor_change)
-		self.treeview.connect("button-press-event", self.do_files_button_press)
+
+		self.treeview.connect("row-activated" ,     self.on_open)
+		self.treeview.connect("cursor-changed",     self.on_cursor_change)
+		self.treeview.connect("button-press-event", self.on_files_button_press)
 
 		scrolled_window = gtk.ScrolledWindow()
 		scrolled_window.add(self.treeview)
@@ -575,18 +661,18 @@ class TopShelf:
 
 		toolbar = gtk.Toolbar()
 
-		self.add_stock_button    (toolbar, gtk.STOCK_OPEN,                            self.do_open       , "Open file")
-		self.add_altstock_button (toolbar, gtk.STOCK_DIRECTORY,      "Open Folder",   self.do_open_folder, "Open folder containing file")
+		self.add_stock_button    (toolbar, gtk.STOCK_OPEN,                            self.on_open       , "Open file")
+		self.add_altstock_button (toolbar, gtk.STOCK_DIRECTORY,      "Open Folder",   self.on_open_folder, "Open folder containing file")
 		toolbar.add(gtk.SeparatorToolItem())
-		self.add_stock_button    (toolbar, gtk.STOCK_GO_UP,                           self.do_move_up    , "Move file up in order")
-		self.add_stock_button    (toolbar, gtk.STOCK_GO_DOWN,                         self.do_move_down  , "Move file down in order")
+		self.add_stock_button    (toolbar, gtk.STOCK_GO_UP,                           self.on_move_up    , "Move file up in order")
+		self.add_stock_button    (toolbar, gtk.STOCK_GO_DOWN,                         self.on_move_down  , "Move file down in order")
 		toolbar.add(gtk.SeparatorToolItem())
-		self.add_stock_button    (toolbar, gtk.STOCK_ADD,                             self.do_add        , "Add a file to the list")
-		self.add_stock_button    (toolbar, gtk.STOCK_REMOVE,                          self.do_remove     , "Remove file from the list")
+		self.add_stock_button    (toolbar, gtk.STOCK_ADD,                             self.on_add        , "Add a file to the list")
+		self.add_stock_button    (toolbar, gtk.STOCK_REMOVE,                          self.on_remove     , "Remove file from the list")
 		toolbar.add(gtk.SeparatorToolItem())
-		self.add_stock_button    (toolbar, gtk.STOCK_HELP,                            self.do_help       , "Show help")
-		self.add_stock_button    (toolbar, gtk.STOCK_ABOUT,                           self.do_about      , "About TopShelf")
-#		self.add_pixbuf_button   (toolbar, guess_mime_icon("a.zip"), "Package Files", self.do_package)
+		self.add_stock_button    (toolbar, gtk.STOCK_HELP,                            self.on_help       , "Show help")
+		self.add_stock_button    (toolbar, gtk.STOCK_ABOUT,                           self.on_about      , "About TopShelf")
+#		self.add_pixbuf_button   (toolbar, guess_mime_icon("a.zip"), "Package Files", self.on_package)
 
 		# Main vbox
 
@@ -601,13 +687,13 @@ class TopShelf:
 		# Popup menu
 
 		self.popup_menu = gtk.Menu()
-		self.add_menuitem   (self.popup_menu, gtk.STOCK_OPEN,                     self.do_open)
-		self.add_altmenuitem(self.popup_menu, gtk.STOCK_DIRECTORY, "Open Folder", self.do_open_folder)
+		self.add_menuitem   (self.popup_menu, gtk.STOCK_OPEN,                     self.on_open)
+		self.add_altmenuitem(self.popup_menu, gtk.STOCK_DIRECTORY, "Open Folder", self.on_open_folder)
 		self.popup_menu.add(gtk.SeparatorMenuItem())
-		self.add_menuitem   (self.popup_menu, gtk.STOCK_GO_UP,                    self.do_move_up)
-		self.add_menuitem   (self.popup_menu, gtk.STOCK_GO_DOWN,                  self.do_move_down)
+		self.add_menuitem   (self.popup_menu, gtk.STOCK_GO_UP,                    self.on_move_up)
+		self.add_menuitem   (self.popup_menu, gtk.STOCK_GO_DOWN,                  self.on_move_down)
 		self.popup_menu.add(gtk.SeparatorMenuItem())
-		self.add_menuitem   (self.popup_menu, gtk.STOCK_REMOVE,                   self.do_remove)
+		self.add_menuitem   (self.popup_menu, gtk.STOCK_REMOVE,                   self.on_remove)
 		self.popup_menu.show_all()
 
 		# Finalize main window
@@ -624,13 +710,20 @@ class TopShelf:
 		btn.set_icon_widget(icon)
 		btn.set_label(alt_text)
 		btn.connect("clicked", callback)
-		btn.set_tooltip_text(tooltip)
+		try:
+			btn.set_tooltip_text(tooltip)
+		except AttributeError:
+			pass # No tooltips, not the end of the world. Only python-gtk2 2.12 has them, i.e., Gutsy and above
+
 		toolbar.insert(btn, -1)
 
 	def add_stock_button(self, toolbar, stock_id, callback, tooltip):
 		btn = gtk.ToolButton(stock_id)
 		btn.connect("clicked", callback)
-		btn.set_tooltip_text(tooltip)
+		try:
+			btn.set_tooltip_text(tooltip)
+		except AttributeError:
+			pass # No tooltips, not the end of the world. Only python-gtk2 2.12 has them, i.e., Gutsy and above
 		toolbar.insert(btn, -1)
 
 	def add_menuitem(self, menu, stock_id, callback):
@@ -721,7 +814,7 @@ class TopShelf:
 			self.files_store.set(iter, FILE_STORE_COLOR,         DISABLE_FG_COLOR)
 			self.files_store.set(iter, FILE_STORE_STRIKETHROUGH, True)
 
-	def check_files_thread(self):
+	def check_all_files(self):
 		if self.pressed:
 			iter = self.files_store.get_iter_first()
 			while iter is not None:
@@ -734,7 +827,11 @@ class TopShelf:
 	# Utilities
 
 	def get_selected(self):
-		return self.treeview.get_selection().get_selected()
+		model, paths = self.treeview.get_selection().get_selected_rows()
+		iters = []
+		for path in paths:
+			iters.append(model.get_iter(path))
+		return model, iters
 
 	def get_n_files(self):
 		ret = self.files_store.iter_n_children(None)
